var AudioComparisonLiteNS = AudioComparisonLiteNS || {};
AudioComparisonLiteNS._construct = function() {
const DE = sessionStorage.getItem("ACL_DEBUG") == "true";
if(typeof BUG != 'function'){
  window.BUG = function(msg){console.debug(msg);};
}
var AC_audio_groups = [];
const AC_SEL_GROUP    = '.audio-comparison-lite';
const AC_SEL_FILE_A   = '.audio-comparison-lite-file-a';
const AC_SEL_FILE_B   = '.audio-comparison-lite-file-b';
const AC_SEL_BTN_PLAY = '.audio-comparison-lite-play-stop';
const AC_SEL_BTN_A    = '.audio-comparison-lite-play-a';
const AC_SEL_BTN_B    = '.audio-comparison-lite-play-b';
const AC_SEL_OUTPUT   = '.audio-comparison-lite-label';
const AC_CLS_LABEL_BUFFERING  = 'audio-comparison-lite-output-buffering';
const AC_CLS_LABEL_PLAYING_A  = 'audio-comparison-lite-output-playing-a';
const AC_CLS_LABEL_PLAYING_B  = 'audio-comparison-lite-output-playing-b';
const AC_DATASET_BUFFERING_TEXT   = 'bufferingText';
const AC_DATASET_BUFFERED_TEXT    = 'bufferedText';
const AC_DATASET_PLAY_DEFAULT     = 'playDefault';
const AC_DATASET_BUTTON_PLAY_TEXT = 'buttonPlayText';
const AC_DATASET_BUTTON_STOP_TEXT = 'buttonStopText';
const AC_DATASET_PLAYING_A_TEXT   = 'playingAText';
const AC_DATASET_PLAYING_B_TEXT   = 'playingBText';
jQuery(document).ready(async function() {
  DE&&BUG('Audio Comparison Lite\n' +
          '---------------------\n' +
          'c.2023 kaedinger\n');
  var groups = jQuery(AC_SEL_GROUP);
  DE&&BUG('Comparison groups found: ' + groups.length);
  jQuery.each(groups, function(gid, group) {
    DE&&BUG('Loading group ' + gid);
    var btnPlay = AC_loadButton(group, AC_SEL_BTN_PLAY);
    var btnA = AC_loadButton(group, AC_SEL_BTN_A);
    var btnB = AC_loadButton(group, AC_SEL_BTN_B);
    if( btnA == null || btnB == null || btnPlay == null ) {
      DE&&BUG('Button A, B or Play missing. Bailing out.');
      return;
    }
    var soundA = AC_loadAudioFile(group, AC_SEL_FILE_A);
    if( soundA == null ) return;
    var soundB = AC_loadAudioFile(group, AC_SEL_FILE_B);
    if( soundB == null ) return;
    var audio_group = {
      jgroup: group,
      btnPlay: btnPlay,
      btnA: btnA,
      btnB: btnB,
      audioA_sound: soundA,
      audioB_sound: soundB,
      lastSeekPos: 0,
      displayTimer: -1
    };
    AC_audio_groups.push(audio_group);
    AC_handleBufferingState(audio_group, false);
    soundA.once('load', function() {
      DE&&BUG('Group ' + gid + ' Sound A has loaded');
      if( soundB.state() === 'loaded' ) {
        AC_handleBufferingState(audio_group, true);
      }
    });
    soundB.once('load', function() {
      DE&&BUG('Group ' + gid + ' Sound B has loaded');
      if( soundA.state() === 'loaded' )
        AC_handleBufferingState(audio_group, true);
    });
  });
  if( AC_audio_groups === null || AC_audio_groups.length <= 0 ) {
    DE&&BUG('No usable comparison groups found!');
    return;
  }
  AudioComparison();
});
function AC_loadAudioFile(group, selector) {
  var selName = selector.substring(1);
  var audio = jQuery(group).find(selector);
  if( audio === null || audio.length <= 0 ) {
    DE&&BUG('No file ' + selName + ' found!');
    return null;
  } else if( audio.length > 1 ) {
    DE&&BUG('Too many files ' + selName + ' found!');
    return null;
  }
  var sound;
  if( audio.is('a') ) {
    sound = audio.attr('href');
    if( !sound ) {
      DE&&BUG('Missing href in link ' + selName);
      return null;
    }
  } else {
    DE&&BUG('No audio file found in ' + selName);
    return null;
  }
  DE&&BUG(selName + ': file ' + sound);
  sound = new Howl({
    src: [sound],
    volume: 1, 
    loop: true,
    html5: true,
    onload: function() { DE&&BUG(selName + ' onload'); },
    onloaderror: function(id, err) { DE&&BUG(selName + ' onloaderror, err=' + err); },
  });
  return sound;
}
function AC_loadButton(group, selector) {
  var btn = jQuery(group).find(selector);
  if( btn === null || btn.length <= 0 ) {
    DE&&BUG('No button ' + selector.substring(1) + 'found!');
    return null;
  }
  return btn;
}
function AC_handleBufferingState(audiogroup, buffered) {
  var label = jQuery(audiogroup.jgroup).find(AC_SEL_OUTPUT);
  if( !buffered ) {
    label
      .text(audiogroup.jgroup.dataset[AC_DATASET_BUFFERING_TEXT] || '')
      .addClass(AC_CLS_LABEL_BUFFERING);
        audiogroup.btnPlay.attr('disabled', true);
      audiogroup.btnA.attr('disabled', true);
      audiogroup.btnB.attr('disabled', true);
  } else {
    label
      .text(audiogroup.jgroup.dataset[AC_DATASET_BUFFERED_TEXT] || '')
      .removeClass(AC_CLS_LABEL_BUFFERING);
      audiogroup.btnPlay.attr('disabled', false);
  }
}
function AudioComparison() {
  jQuery.each(AC_audio_groups, function(aid, audio) {
      DE&&BUG('Setting group ' + aid + ' play button click handlers');
      jQuery.each(audio.btnPlay, function(bid, btn) {
        jQuery(btn).click({group: aid, btn: bid}, AC_btnPlay_Clicked);
      });
      DE&&BUG('Setting group ' + aid + ' A button click handlers');
      jQuery.each(audio.btnA, function(bid, btn) {
        jQuery(btn).click({group: aid, btn: bid}, AC_btnA_Clicked);
      });
      DE&&BUG('Setting group ' + aid + ' B button click handlers');
      jQuery.each(audio.btnB, function(bid, btn) {
        jQuery(btn).click({group: aid, btn: bid}, AC_btnB_Clicked);
      });
  });
}
function AC_btnPlay_Clicked(event) {
  DE&&BUG('Group ' + event.data.group + ' play button ' + event.data.btn + ' was clicked');
  var gid = event.data.group;
  var group = AC_audio_groups[gid];
  let playDefault = group.jgroup.dataset[AC_DATASET_PLAY_DEFAULT] || 'a';
  var playTrackA = playDefault.toLowerCase() == "a";
  if( group.audioA_sound.playing() || group.audioB_sound.playing() ) {
    AC_stopGroup(group);
    group.audioA_sound.off('end');
      group.btnPlay.text(group.jgroup.dataset[AC_DATASET_BUTTON_PLAY_TEXT] || 'Play');
      group.btnA.attr('disabled','true');
      group.btnB.attr('disabled','true');
    jQuery(group.jgroup).find(AC_SEL_OUTPUT)
      .text('').removeClass(AC_CLS_LABEL_PLAYING_A).removeClass(AC_CLS_LABEL_PLAYING_B);
  } else {
      group.btnPlay.text(group.jgroup.dataset[AC_DATASET_BUTTON_STOP_TEXT] || 'Stop');
    AC_stopGroup(group);
    if( playTrackA ) {
      AC_playA(group);
    } else {
      AC_playB(group);
    }
  }
}
function AC_switchTracks(group, playTrackA) {
  DE&&BUG('Switch tracks, playTrackA = ' + playTrackA);
  playTrackA = playTrackA || group.audioB_sound.playing();
  DE&&BUG('-> playTrackA = ' + playTrackA);
  AC_stopGroup(group);
  if( playTrackA ) {
    DE&&BUG('Switching tracks: B -> A');
    AC_playA(group);
  } else {
    DE&&BUG('Switching tracks: A -> B');
    AC_playB(group);
  }
}
function AC_btnA_Clicked(event) {
  DE&&BUG('Group ' + event.data.group + ' button A ' + event.data.btn + ' was clicked');
  var group = AC_audio_groups[event.data.group];
    AC_switchTracks(group, true);
}
function AC_btnB_Clicked(event) {
  DE&&BUG('Group ' + event.data.group + ' button B ' + event.data.btn + ' was clicked');
  var group = AC_audio_groups[event.data.group];
    AC_switchTracks(group, false);
}
function AC_playA(group) {
    group.btnA.attr('disabled','true');
    group.btnB.removeAttr('disabled');
  jQuery(group.jgroup).find(AC_SEL_OUTPUT)
    .addClass(AC_CLS_LABEL_PLAYING_A).removeClass(AC_CLS_LABEL_PLAYING_B);
  group.audioA_sound.seek(group.lastSeekPos);
  group.audioA_sound.play();
   AC_displayPlayingTextHelper(group, group.jgroup.dataset[AC_DATASET_PLAYING_A_TEXT] || '', group.lastSeekPos);
   group.displayTimer = setInterval(AC_displayPlayingText, 100, group);
}
function AC_playB(group) {
    group.btnB.attr('disabled','true');
    group.btnA.removeAttr('disabled');
  jQuery(group.jgroup).find(AC_SEL_OUTPUT)
    .addClass(AC_CLS_LABEL_PLAYING_B).removeClass(AC_CLS_LABEL_PLAYING_A);
  group.audioB_sound.seek(group.lastSeekPos);
  group.audioB_sound.play();
  AC_displayPlayingTextHelper(group, group.jgroup.dataset[AC_DATASET_PLAYING_B_TEXT] || '', group.lastSeekPos);
  group.displayTimer = setInterval(AC_displayPlayingText, 100, group);
}
function AC_stopGroup(group) {
  if( group.audioA_sound.playing() ) {
    group.lastSeekPos = group.audioA_sound.seek();
    group.audioA_sound.stop();
  } 
  if( group.audioB_sound.playing() ) {
    group.lastSeekPos = group.audioB_sound.seek();
    group.audioB_sound.stop();
  }
  if(group.displayTimer > -1) {
    clearInterval(group.displayTimer);
    group.displayTimer = -1;
  }
}
function AC_displayPlayingText(group) {
  let text = '';
  let seek = 0;
  if( group.audioA_sound.playing() ) {
    seek = group.audioA_sound.seek();
    text = group.jgroup.dataset[AC_DATASET_PLAYING_A_TEXT] || '';
  } 
  if( group.audioB_sound.playing() ) {
    seek = group.audioB_sound.seek();
    text = group.jgroup.dataset[AC_DATASET_PLAYING_B_TEXT] || '';
  }
  AC_displayPlayingTextHelper(group,text,seek);
}
function AC_displayPlayingTextHelper(group,text,seek) {
  let dtime = (new Date(0,0,0,0,0,seek)).toTimeString().substring(3,8);
  jQuery(group.jgroup).find(AC_SEL_OUTPUT)
    .text(text.replace("%T", dtime));
}
}
AudioComparisonLiteNS._construct();
