<?php
/*
  Plugin Name: Audio Comparison Lite
  Plugin URI: https://audiocomparison.kaedinger.de/lite
  Description: Time synchronized A/B comparison of two audio files side by side. In Style. For detailed documentation, please visit the plugin's <a href="https://audiocomparison.kaedinger.de/"> official page</a>.
  Version: 1.9
  Author: kaedinger
  Author URI: https://kaedinger.de
  License: GPLv2 or later 
*/
defined( 'ABSPATH' ) || exit;
if (!function_exists('AC_write_log')) {
  function AC_write_log($log) {
    if (true === WP_DEBUG) {
      if (is_array($log) || is_object($log)) {
          $log = print_r($log, true);
      }
      error_log("Audio Comparison: $log");
    }
  }
}
if( ! class_exists( 'audioComparisonLite' ) ) {
  class audioComparisonLite {
    const SHORTCODE = 'audiocomparisonlite';
    const MAIN_DIV = 'audio-comparison-lite';
    const OPTIONS_KEY = 'audiocomparisonlite_options';
    const SETTINGS_TITLE = 'Audio Comparison Lite';
    const LITE = 'LITE ';
    public function __construct() {
      add_action( 'wp_enqueue_scripts', array($this, 'enqueue_scripts' ) );
      add_action( 'init', array($this, 'init' ) );
      add_action('admin_menu', array($this, 'create_menu' ) );
      add_action('admin_init', array($this, 'admin_init' ));
    }
    public function enqueue_scripts() {
      wp_enqueue_script('jquery');
      wp_enqueue_script('howler-js', plugins_url('lib/howler.core.min.js', __FILE__));
      wp_enqueue_script('audio-comparison-lite', plugins_url('audio-comparison.js', __FILE__), array('jquery', 'howler-js'));
    }
    public function init() {
      add_shortcode(self::SHORTCODE, array($this, 'shortcode' ) );
    }
    public function shortcode($attr = [], $content = null, $tag = '') {
      $plugin_data = $this->get_plugin_data();
      $opts = $this->get_options();
      if ($opts['text_ready'] === "debug")
      {
        $buff = '<h3>Audio Comparison is in debug mode!</h3>';
        $buff .= '<h4>plugin_data</h4>';
        $buff .= '<p><pre>' . print_r($plugin_data, true) . '</pre></p>';
        $buff .= '<h4>opts</h4>';
        $buff .= '<p><pre>' . print_r($opts, true) . '</pre></p>';
      }
      $attr = array_change_key_case( (array) $attr, CASE_LOWER );
      if ($opts['text_ready'] === "debug")
      {
        $buff .= '<h4>lowercase attr</h4>';
        $buff .= '<p><pre>' . print_r($attr, true) . '</pre></p>';
      }
      $attr = shortcode_atts(
        array(
          'play_button' => $opts['play_button'],
          'text_button_play' => $opts['text_button_play'],
          'text_button_stop' => $opts['text_button_stop'],
          'text_a' => $opts['text_a'],
          'text_b' => $opts['text_b'],
          'text' => $opts['text'],
          'text_loading' => $opts['text_loading'],
          'text_ready' => $opts['text_ready'],
          'text_button_a' => $opts['text_button_a'],
          'text_button_b' => $opts['text_button_b'],
          'before' => '',
          'file_a' => '',
          'file_b' => '',
          'play_default' => $opts['play_default'],
        ), $attr, $tag
      );
      if ($opts['text_ready'] === "debug")
      {
        $buff .= '<h4>shortcode_atts</h4>';
        $buff .= '<p><pre>' . print_r($attr, true) . '</pre></p>';
        return $buff;
      }
      $endl = "\n";
      $playbuttonbuff = '';
      $playbuttontextsbuff = '';
      $playButton = esc_html($attr['play_button']);
      {
        $playDefault = esc_html($attr['play_default']);
        $playbuttontextsbuff .= ' data-play-default="' . $playDefault . '"';
        $textPlay = esc_html($attr['text_button_play']);
        $textStop = esc_html($attr['text_button_stop']);
        $playbuttontextsbuff .= ' data-button-play-text="' . $textPlay . '"';
        $playbuttontextsbuff .= ' data-button-stop-text="' . $textStop . '"';
        $playbuttonbuff .= '<button class="' . self::MAIN_DIV . '-play-stop">' . $textPlay . '</button>' . $endl;
      }
      $buff = '';
      $buff .= "\n<!-- {$plugin_data['name']} | {$plugin_data['url']} -->\n";
      $buff .= '<div class="' . self::MAIN_DIV . '"';
      $text = esc_html($attr['text']);
      {
        $textA = esc_html($attr['text_a']);
        if(!empty($textA)) $buff .= ' data-playing-a-text="' . $textA . '"';
        $textB = esc_html($attr['text_b']);
        if(!empty($textB)) $buff .= ' data-playing-b-text="' . $textB . '"';
        $textLoading = esc_html($attr['text_loading']);
        if(!empty($textLoading)) $buff .= ' data-buffering-text="' . $textLoading . '"';
        $textReady = esc_html($attr['text_ready']);
        if(!empty($textReady)) $buff .= ' data-buffered-text="' . $textReady . '"';
      }
      $buff .= $playbuttontextsbuff . ">\n";
      $buff .= esc_html($attr['before']);
      $label = '<span class="' . self::MAIN_DIV . '-label"></span>' . $endl;
      if ($text == "before")
      {
        $buff .= $label;
        $label = '';
      }
      $fileA = esc_html($attr['file_a']);
      if(!empty($fileA)) $buff .= '<a class="' . self::MAIN_DIV . '-file-a" href="' . $fileA . '"></a>' . $endl;
      $fileB = esc_html($attr['file_b']);
      if(!empty($fileB)) $buff .= '<a class="' . self::MAIN_DIV . '-file-b" href="' . $fileB . '"></a>' . $endl;
      if($playButton == "left")
      {
        $buff .= $playbuttonbuff;
        $playbuttonbuff = '';
      }
      $buttonA = esc_html($attr['text_button_a']);
      $buff .= '<button class="' . self::MAIN_DIV . '-play-a">' . $buttonA . '</button>' . $endl;
      $buttonB = esc_html($attr['text_button_b']);
      $buff .= '<button class="' . self::MAIN_DIV . '-play-b">' . $buttonB . '</button>' . $endl;
      $buff .= $playbuttonbuff;
      $buff .= $label;
      $buff .= "</div><!-- /" . self::MAIN_DIV . " -->\n";
      $buff .= "<!-- {$plugin_data['name']} | {$plugin_data['url']} -->\n";
      return $buff;
    }
    public function get_plugin_data() {
      $default_headers = array(
          'Name' => 'Plugin Name',
          'PluginURI' => 'Plugin URI',
      );
      $plugin_data = get_file_data(__FILE__, $default_headers, 'plugin');
      $url = $plugin_data['PluginURI'];
      $name = $plugin_data['Name'];
      $data['name'] = $name;
      $data['url'] = $url;
      return $data;
    }
    public function get_options() {
      $defaults = array(
          'text' => 'before',            
          'play_button' => 'left',       
          'text_a' => 'Now playing A',
          'text_b' => 'Now playing B',
          'text_loading' => 'Buffering audio...',
          'text_ready' => 'Ready!',
          'text_button_play' => 'Play',
          'text_button_stop' => 'Stop',
          'text_button_a' => 'Version A',
          'text_button_b' => 'Version B',
          'play_default' => 'A',         
      );
      $current_options = get_option( self::OPTIONS_KEY, $defaults);
      $current_options = array_merge($defaults, $current_options);
      return $current_options;
    }
    public function set_options($opts) {
      foreach ($opts as $key => $value) {
          $value = wp_kses($value, array());
          $value = trim($value);
          $opts[$key] = $value;
      }
      update_option( self::OPTIONS_KEY , $opts);
      return $opts;
    }
    public function create_menu() {
      add_options_page(self::SETTINGS_TITLE, self::SETTINGS_TITLE, 'manage_options', __FILE__, array($this, 'settings_page' ) );
      add_filter('plugin_action_links', array($this, 'add_plugin_settings_link' ), 10, 2);
      add_filter( 'plugin_row_meta', [ $this, 'add_plugin_row_meta_link' ], 10, 2 );
    }
    public function add_plugin_row_meta_link($links, $file) {
      if ($file == plugin_basename(__FILE__)) {
        $doc_link = '<a href="https://audiocomparison.kaedinger.de/docs" target="_blank" rel="noopener noreferrer">Documentation</a>';
        $links[] = $doc_link;
      }
      return $links;
    }
    public function add_plugin_settings_link($links, $file) {
      if ($file == plugin_basename(__FILE__)) {
        $link = admin_url('options-general.php?page=' . plugin_basename(__FILE__));
        $dashboard_link = "<a href=\"{$link}\">Settings</a>";
        array_unshift($links, $dashboard_link);
      }
      return $links;
    }
    public function settings_page() {
      $saved = 0;
      $current_options = $this->get_options();
      $settings_hint = '';
      if (!empty($_POST)) 
      {
        $current_options_keys = array_keys($current_options);
        $_REQ = stripslashes_deep($_REQUEST);
        foreach ($_REQ as $key => $value) {
          if (in_array($key, $current_options_keys)) {
            $value = wp_kses($value, array());
            $value = trim($value);
            if(strpos($value, "*") !== false) {
              $settings_hint = '<br>* Options are exclusive to the full version of Audio Comparison. See <a href="https://audiocomparison.kaedinger.de">https://audiocomparison.kaedinger.de</a>';
            } else {
              $current_options[$key] = $value;
            }
          }
        }
        $current_options = $this->set_options($current_options);
        $saved = 1;
      }
    ?>
    <div class="wrap <?php echo self::SHORTCODE; ?>_container">
      <style>
      #<?php echo self::SHORTCODE; ?>_settings th {
        text-align: right;
      }
      #<?php echo self::SHORTCODE; ?>_settings select {
        width: 25ch;
      }
      #<?php echo self::SHORTCODE; ?>_settings input {
        width: 20ch;
      }
      #<?php echo self::SHORTCODE; ?>_settings img {
        width: 200px;
      }
      </style>
    <div>
      <div style="float: left">
        <img src="<?php echo plugins_url('/logo.png', __FILE__); ?>" style="width: 100px">
      </div>  
      <div style="margin-left: 110px">
        <p><span style="font-size: x-large"><?php echo self::SETTINGS_TITLE; ?></span></p>
          <p>
            Time synchronized A/B comparison <?php echo self::LITE; ?>of two audio files side by side. In style.<br>
            Developed by <a href="https://studio.kaedinger.de">studio kaedinger</a>
            &bull; Full version/documentation: <a href="https://audiocomparison.kaedinger.de">https://audiocomparison.kaedinger.de</a>
            &bull; <strong><a href="https://audiocomparison.kaedinger.de">Subscription and perpetual license available!</a></strong>
          </p>  
        </div>  
      </div>  
      <h2>Settings</h2>
      <?php if (!empty($saved)) : ?>
            <div class="updated">
                <p>Settings were saved.<?php echo wp_kses_post( $settings_hint ); ?></p>
            </div>
      <?php endif; ?>
      <form method="post">
        <table id="<?php echo self::SHORTCODE; ?>_settings">
          <tr>
            <th scope="row"><label for="text">Output:</label></th>
            <td><select id="text" name="text">
                  <option value="before"<?php echo wp_kses_post( $current_options['text'] == "before" ? " selected" : "" ); ?>>Before the buttons</option>
                  <option value="after"<?php echo wp_kses_post( $current_options['text'] == "after" ? " selected" : "" ); ?>>After the buttons</option>
                  <option value="*none">* No text output</option>
                </select>
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="text_a">Output for A:</label></th>
            <td><input maxlength="30" id="text_a" name="text_a"
                                      autocomplete="off"
                                      value="<?php echo esc_attr($current_options['text_a']); ?>" />
                  e.g. 'Now playing A'
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="text_b">Output for B:</label></th>
            <td><input maxlength="30" id="text_b" name="text_b"
                                      autocomplete="off"
                                      value="<?php echo esc_attr($current_options['text_b']); ?>" />
                  e.g. 'Now playing B'
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="text_loading">Output during file load:</label></th>
            <td><input maxlength="30" id="text_loading" name="text_loading"
                                      autocomplete="off"
                                      value="<?php echo esc_attr($current_options['text_loading']); ?>" />
                  e.g. 'Buffering audio...'
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="text_ready">Output after file load:</label></th>
            <td><input maxlength="30" id="text_ready" name="text_ready"
                                      autocomplete="off"
                                      value="<?php echo esc_attr($current_options['text_ready']); ?>" />
                  e.g. 'Ready!'
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="play_button">Play/Stop button:</label></th>
            <td><select id="play_button" name="play_button">
                  <option value="left"<?php echo wp_kses_post( $current_options['play_button'] == "left" ? " selected" : "" ); ?>>Left of the AB buttons</option>
                  <option value="right"<?php echo wp_kses_post( $current_options['play_button'] == "right" ? " selected" : "" ); ?>>Right of the AB buttons</option>
                  <option value="*none">* No play button</option>
                </select>
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="play_default">Play default:</label></th>
            <td><select id="play_default" name="play_default">
                  <option value="A"<?php echo wp_kses_post( $current_options['play_default'] == "A" ? " selected" : "" ); ?>>Play A on default</option>
                  <option value="B"<?php echo wp_kses_post( $current_options['play_default'] == "B" ? " selected" : "" ); ?>>Play B on default</option>
                </select>
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="text_button_play">Label on Play/Stop button for Play:</label></th>
            <td><input maxlength="30" id="text_button_play" name="text_button_play"
                                      autocomplete="off"
                                      value="<?php echo esc_attr($current_options['text_button_play']); ?>" />
                  e.g. 'Play'
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="text_button_stop">Label on Play/Stop button for Stop:</label></th>
            <td><input maxlength="30" id="text_button_stop" name="text_button_stop"
                                      autocomplete="off"
                                      value="<?php echo esc_attr($current_options['text_button_stop']); ?>" />
                  e.g. 'Stop'
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="text_button_a">Label on A button:</label></th>
            <td><input maxlength="30" id="text_button_a" name="text_button_a"
                                      autocomplete="off"
                                      value="<?php echo esc_attr($current_options['text_button_a']); ?>" />
                  e.g. 'Version A'
            </td>
          </tr>
          <tr>
            <th scope="row"><label for="text_button_b">Label on B button:</label></th>
            <td><input maxlength="30" id="text_button_b" name="text_button_b"
                                      autocomplete="off"
                                      value="<?php echo esc_attr($current_options['text_button_b']); ?>" />
                  e.g. 'Version B'
            </td>
          </tr>
          <tr>
            <td>
              <br><input type="submit" name="save_settings" value="Save Changes" class="button-primary" />
            </td>
          </tr>
        </table>
      </form>
      <h2>Shortcode</h2>
      <p>The base version of <?php echo self::SETTINGS_TITLE; ?>'s shortcode is <code>[<?php echo self::SHORTCODE; ?> file_a="PATH/TO/AUDIO.A" file_b="PATH/TO/AUDIO.B"]</code><br>
      This way, all settings from above are used. It's a great way to keep a consistent look throughout your website for all instances of <?php echo self::SETTINGS_TITLE; ?>.</p>
      <p>However, you can give any instance an individual UI. Any option from above is also available as a shortcode attribute. 
        Any attribute not specified in the shortcode will use the site-wide setting from above.</p>
      <table>
      <tr>
          <td><code>text</code></td>
          <td><strong>Output</strong>. Sets the position of the output label. Options: <code>before</code> (before the buttons), <code>after</code> (after the buttons), * <code>none</code> (no output label; <a href="https://audiocomparison.kaedinger.de">only available in the full version of Audio Comparison</a>). Default: <code>before</code></td>
        </tr>
        <tr>
          <td><code>text_a</code></td>
          <td><strong>Output for A</strong>. Contents of output label while playing audio A. Default: <code>Now playing A</code></td>
        </tr>
        <tr>
          <td><code>text_b</code></td>
          <td><strong>Output for B</strong>. Contents of output label while playing audio B. Default: <code>Now playing B</code></td>
        </tr>
        <tr>
          <td><code>text_loading</code></td>
          <td><strong>Output during file load</strong>. Contents of output label while buffering audio. Default: <code>Buffering audio...</code></td>
        </tr>
        <tr>
          <td><code>text_ready</code></td>
          <td><strong>Output after file load</strong>. Contents of output label after audio has been loaded. Default: <code>Ready!</code></td>
        </tr>
        <tr>
          <td><code>play_button</code></td>
          <td><strong>Play/Stop button</strong>. Sets the position of the Play/Stop button. Options: <code>left</code> (left of the A/B buttons), <code>right</code> (right of the A/B buttons), * <code>none</code> (no Play/Stop button; <a href="https://audiocomparison.kaedinger.de">only available in the full version of Audio Comparison</a>). Default: <code>left</code></td>
        </tr>
        <tr>
          <td><code>play_default</code></td>
          <td><strong>Play on default</strong>. What sound to play after pressing the Play button. Options: <code>A</code> (sound A), <code>B</code> (sound B). Default: <code>A</code></td>
        </tr>
        <tr>
          <td><code>text_button_play</code></td>
          <td><strong>Label on Play/Stop button for Play</strong>. Play/Stop button label while not playing audio. Default: <code>Play</code></td>
        </tr>
        <tr>
          <td><code>text_button_stop</code></td>
          <td><strong>Label on Play/Stop button for Stop</strong>. Play/Stop button label while playing audio. Default: <code>Stop</code></td>
        </tr>
        <tr>
          <td><code>text_button_a</code></td>
          <td><strong>Label on A button</strong>. Text on button A. Default: <code>Version A</code></td>
        </tr>
        <tr>
          <td><code>text_button_b</code></td>
          <td><strong>Label on B button</strong>. Text on button B. Default: <code>Version B</code></td>
        </tr>
      </table>
      <h2>Design</h2>
      <p>You can use the following CSS classes to fine tune the visual style of the label.</p>
      <h4>Static</h4>
      <table>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?></code></td>
          <td>The whole instance <code>div</code>.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-play-stop</code></td>
          <td>Play/Stop button.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-play-a</code></td>
          <td>Button A.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-play-b</code></td>
          <td>Button B.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-label</code></td>
          <td>Output label.</td>
        </tr>
      </table>
      <h4>Dynamic</h4>
      <table>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-output-buffering</code></td>
          <td>Assigned to the output label while audio is still loading.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-output-playing-a</code></td>
          <td>Assigned to the output label while audio A is playing.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-output-playing-b</code></td>
          <td>Assigned to the output label while audio B is playing.</td>
        </tr>
        <tr>
          <td><code>disabled="disabled"</code></td>
          <td >Buttons that cannot be clicked<br>(i.e. Play/Stop 
            button during buffering, button A while playing A, button B while playing B)<br>
            have the <code>disabled</code> attribute set.</td>
        </tr>
      </table>
    </div>
      <?php
    }
    public function admin_init() {
      $this->load_assets();
    }
    public function load_assets() {
      wp_enqueue_script('jquery');
    }
  }
  new audioComparisonLite();
}
// promotion checker
if( ! class_exists( 'audioComparisonLitePromotionChecker' ) ) {
	class audioComparisonLitePromotionChecker{
		public $cache_key;
		public $cache_allowed;
		public function __construct() {
			$this->cache_key = 'audio_comparison_lite_info';
			$this->cache_allowed = false;
      add_filter( 'plugin_row_meta', [ $this, 'add_plugin_row_meta_link' ], 11, 2 );
		}
    public function add_plugin_row_meta_link($links, $file) {
      if ($file == plugin_basename(__FILE__)) {
        $mylinks = $this->get_links();
        if(count($mylinks) == 0) {
          $pro_link = '<a href="https://audiocomparison.kaedinger.de" 
          target="_blank" rel="noopener noreferrer" 
          style="background-color: #f0a07c; color: #4a274f; font-weight: 700" 
          onmouseover="this.style.color=\'#f0a07c\'; this.style.backgroundColor=\'#4a274f\';" 
          onmouseout="this.style.color=\'#4a274f\'; this.style.backgroundColor=\'#f0a07c\';" 
          >&nbsp;Get Pro!&nbsp;</a>';
          $mylinks[] = $pro_link;
        }
        $links = array_merge($links, $mylinks);
      }
      return $links;
    }
		function request() {
      AC_write_log("get_transient");
			$remote = get_transient( $this->cache_key );
			if( false === $remote || ! $this->cache_allowed ) {
        AC_write_log("wp_remote_get");
				$remote = wp_remote_get(
					'https://assets.kaedinger.de/static/ACL/infoACL.json',
					array(
						'timeout' => 10,
						'headers' => array(
							'Accept' => 'application/json'
						)
					)
				);
        AC_write_log($remote);
				if(
					is_wp_error( $remote )
					|| 200 !== wp_remote_retrieve_response_code( $remote )
					|| empty( wp_remote_retrieve_body( $remote ) )
				) {
					return false;
				}
				set_transient( $this->cache_key, $remote, DAY_IN_SECONDS );
			}
			$remote = json_decode( wp_remote_retrieve_body( $remote ), true );
			return $remote;
		}
		function get_links() {
      $res = [];
			$remote = $this->request();
			if( ! $remote ) {
				return $res;
			}
      if(array_key_exists('numlinks', $remote)) {
        for($i = 0; $i < $remote['numlinks']; $i++) {
          if(array_key_exists("link$i", $remote)) {
            $entry = $remote["link$i"];
            $now = new DateTime();
            $date = DateTime::createFromFormat('Y-m-d', $entry["from"]);
            if($date > $now) continue;
            $date = DateTime::createFromFormat('Y-m-d', $entry["to"]);
            if($date < $now) continue;
            $res[] = $entry["link"];
          }
        }
      }
			return $res;
		}
	}
	new audioComparisonLitePromotionChecker();
}
